
#include <evblib/evproc/evproc.h>

#include <stdlib.h>			/* for free */
#include <string.h>			/* for memset, memcpy */


/* local functions */

static void evprocbox_del( EVPROCBOX_T *epb )
{
    switch( epb->endstyle )
    {
	case EP_REM_NOP:
	    break;

	case EP_REM_CALLDONE:
	    epb->ep->ep_done( epb->ep );
	    break;

	case EP_REM_CALLDONE_FREE:
	    epb->ep->ep_done( epb->ep );
	    free( epb->ep );
	    break;
    }
    free( epb );
}


/* the empty function bodies for the null evproc interface */
 
#define F(type, name, args...)	type null_##name( args ) { return (type)0; }
#include <evblib/evproc/evproc.inc>
#undef F


/* Evprocset */

/* constructor */

void evprocset_init( EVPROCSET_T *eps )
{
    memset( eps, 0, sizeof( *eps ) );
    evprocset_ep_init( &eps->ep );
    handleset_init( &eps->cachedhs );
}

/* evproc interface */

void evprocset_ep_init( EVPROC_T *ep )
{
    memset( ep, 0, sizeof( *ep ) );

    ep->ep_addhandles	    = evprocset_ep_addhandles;
    ep->ep_handle	    = evprocset_ep_handle;
    ep->ep_done		    = null_ep_done;
}

void evprocset_ep_addhandles( EVPROC_T *ep, HANDLESET_T *io_hs )
{
    EVPROCSET_T *eps = (EVPROCSET_T *)ep;		/* downcast */

    handleset_addhs( io_hs, &eps->cachedhs );
}

int evprocset_ep_handle( EVPROC_T *ep, HANDLESET_T *hs )
{
    EVPROCSET_T *eps = (EVPROCSET_T *)ep;		/* downcast */
    EVPROCBOX_T *epb, *nextepb;
    int res;

    /* call ep_handle with the handleset for all member evprocs, removing all
     * evprocs that have exited in the manner specified when adding them to 
     * the list */

    for( epb = eps->ephead; epb; epb = nextepb )
    {
	nextepb = epb->next;

	res = epb->ep->ep_handle( epb->ep, hs );
	if ( res <= 0 ) evprocset_rem( eps, epb );
    }

    /* return true as long as we have any left; note that ep_handle may have
     * side effects on the evprocset, so we need to retest for emptyness here */

    return eps->ephead != 0;
}

/* evprocset's own interface */

EVPROCBOX_T *evprocset_add( EVPROCSET_T *eps, EVPROC_T *ep, 
			    evprocbox_endstyle_t endstyle )
{
    EVPROCBOX_T *epb;

    epb = (EVPROCBOX_T *)malloc( sizeof( *epb ) );
    if ( epb == 0 ) return epb;

    epb->ep	    = ep;
    epb->endstyle   = endstyle;

    LIST_ADDTAIL( &eps->ephead, &eps->eptail, epb );

    /* no need to walk the whole eps list; just add the handles to the cache */

    ep->ep_addhandles( ep, &eps->cachedhs );

    return epb;
}

void evprocset_rem( EVPROCSET_T *eps, EVPROCBOX_T *epb )
{
    /* remove the box from the list */

    LIST_DEL( &eps->ephead, &eps->eptail, epb );
    evprocset_update( eps );

    /* properly dispose of the box and its contents */

    evprocbox_del( epb );
}

void evprocset_update( EVPROCSET_T *eps )
{
    EVPROCBOX_T *epb;

    handleset_clear( &eps->cachedhs );
    for( epb = eps->ephead; epb; epb = epb->next )
    {
	epb->ep->ep_addhandles( epb->ep, &eps->cachedhs );
    }
}

void evprocset_clear( EVPROCSET_T *eps )
{
    EVPROCBOX_T *epb, *prevepb;

    for( epb = eps->eptail; epb; epb = prevepb ) 
    {
	prevepb = epb->prev;
	evprocbox_del( epb );
    }
    eps->ephead = eps->eptail = 0;
    handleset_clear( &eps->cachedhs );
}

void evprocset_done( EVPROCSET_T *eps )
{
    evprocset_init( eps );
}


/*
 * vim:softtabstop=4:sw=4
 */

