/*
 * FASTRING - support for fast ring buffers using the mirror page technique
 *
 * Author:
 * Emile van Bergen, emile@e-advies.nl
 *
 * License:
 * Permission to redistribute an original or modified version of this program
 * in source, intermediate or object code form is hereby granted exclusively
 * under the terms of the GNU General Public License, version 2. Please see the
 * file COPYING for details, or refer to http://www.gnu.org/copyleft/gpl.html.
 *
 * History: 
 * 2003/05/08 - EvB - Created
 */

char fastring_id[] = "FASTRING - Copyright (C) 2003 Emile van Bergen.";


#include <stdlib.h>		/* for malloc, free */
#include <string.h>		/* for memset */

#include <evblib/fastring/fastring.h>


RING_T *ring_new(size_t size)
{
    RING_T *ret;

    ret = (RING_T *)malloc(sizeof(RING_T)); 
    if (!ret) return 0;

    if (ring_init(ret, size) < 0) { free(ret); ret = 0; }

    return ret;
}

int ring_init(RING_T *r, size_t size)
{
    int ret;

    memset(r, 0, sizeof(*r));
    ret = mirr_init(&r->mp, size);
    r->r = r->w = r->mp.p;
    return ret;
}

void ring_done(RING_T *r)
{
    mirr_done(&r->mp);
}

void ring_del(RING_T *r)
{
    ring_done(r);
    free(r);
}


/* 
 * vim:softtabstop=4:sw=4
 */

