#ifndef _MISC_H
#define _MISC_H 1


/* Macros and prototypes for miscellaneous support functions */


#include <sys/types.h>			/* for size_t */

#include <evblib/sysdefs/byteorder.h>


/* Min/Max macros */

#define MIN(a, b)	((a) < (b) ? (a) : (b))
#define MAX(a, b)	((a) > (b) ? (a) : (b))


/* Array element count */

#define DIM(arr)	(sizeof(arr)/sizeof(arr[0]))


/* Abnormally terminate program with message given by string constant */

#define die(cstr)	\
	do { if (sizeof(cstr) > 1) write(2, pstr(cstr));	\
	     _exit(127); } while(0)

/* Ensure condition is true; otherwise exit with message */

#define die_unless(cond, cstr)	\
	if (!(cond)) die(cstr)


/* Allocate memory and exit immediately if that fails */

void *safe_malloc(size_t len);


/* A read that never returns -1. EINTR is handled, the rest ends the program. 
   Not suitable for nonblocking I/O, unless you can guarantee that no EGAIN
   will occur, which is hard and failure prone */

int safe_read(int fd, void *p, size_t l);


/* Get random data from RANDOM_DEV */

void get_random_data(char *p, ssize_t len);


/* Get a fixed length bigendian value from a byte buffer */

static inline uint16_t getnet16(void *buf)
{
	unsigned char *b = (unsigned char *)buf;
	return b[0] << 8 | b[1];
}

static inline uint32_t getnet32(void *buf)
{
	unsigned char *b = (unsigned char *)buf;
	return b[0] << 24 | b[1] << 16 | b[2] << 8 | b[3];
}


/* Transform an octet string into its hexadecimal representation */

void hex(char *buf, void *src, ssize_t len);

/* Convert values into their hexadecimal representation. Not macros,
 * because that would either require modifyable lvalue arguments or
 * local variables, which aren't widely supported enough. */

static inline void hex8(char *buf, char val)
{
    hex(buf, &val, 1);
}

static inline void hex16(char *buf, uint16_t val)
{
    val = net16(val);
    hex(buf, &val, 2);
}

static inline void hex32(char *buf, uint32_t val)
{
    val = net32(val);
    hex(buf, &val, 4);
}


/* Print a hexdump of a buffer to a file descriptor */

void hexdumpfd(int fd, void *buf, ssize_t len, ssize_t dispofs);


#endif


/*
 * vim:softtabstop=4:sw=4
 */

