
#include <evblib/evproc/evproc.h>

#include <stdio.h>
#include <errno.h>

/* Generate evproc interface for myevproc */

#define F(type, name, args...)	type myevproc_##name( args );
#include <evblib/evproc/evproc.inc>
#undef F

void myevproc_init( EVPROC_T *ep, FD_T fd )
{
	ep->ep_init = null_ep_init;
	ep->ep_addhandles = myevproc_ep_addhandles;
	ep->ep_handle = myevproc_ep_handle;
	ep->ep_done = null_ep_done;

	ep->private = (void *)fd;
}

void myevproc_ep_addhandles( EVPROC_T *ep, HANDLESET_T *io_hs )
{
	handleset_addfd( io_hs, (FD_T)ep->private, 0 );
}

int myevproc_ep_handle( EVPROC_T *ep, HANDLESET_T *hs )
{
	int n, ret;
	unsigned char c;

	ret = 0;
	if ( handleset_hasfd( hs, (FD_T)ep->private, 0 ) )
	{
		c = 0;
		n = read( (int)ep->private, &c, 1 );
		printf( "read 1 from %d returned %d - %3d (%02x) '%c'\n", (int)ep->private,
				n, c, c, c > 31 && c < 127 ? c : '.' );
		if ( n <= 0 && errno != EAGAIN )
		{
			ep->private = (void *)-1;
			ret = 1;
		}
	}
	return ret;
}

int myevproc_isdone( EVPROC_T *ep )
{
	return (int)ep->private < 0;
}

int main()
{
	EVPROC_T ep;
	HANDLESET_T pendingset, waitset;
	TIMER_T t;
	int res;

	myevproc_init( &ep, 0 );

	handleset_init( &waitset );
	myevproc_ep_addhandles( &ep, &waitset );

	printf( "Type lines to test, done with ^D\n");

	timer_now( &t );
	while( myevproc_isdone( &ep ) == 0 )
	{
		pendingset = waitset;
		res = handleset_wait( &pendingset, &t );
		if ( res < 0 ) { errno = -res; perror( "handle_wait" ); break; }

		if ( myevproc_ep_handle( &ep, &pendingset ) )
		{
			handleset_init( &waitset );
			myevproc_ep_addhandles( &ep, &waitset );
		}
	}

	return 0;
}

/*
 * vim:ts=4:sw=4
 */
