
/* This is the UVL/UTF-8 encoder routine body. You need to prepend a function
 * declaration that contains at least the following arguments:
 *
 *	int utf8fromwhatever(char *d, int dl)
 *
 * and you need to define three macros to access whatever source parameters you
 * add: UTEST to check if there is input remaining to get, UGET to get one
 * unicode code point; UADV to advance the input stream, and UDONE to 
 * optionally set a return parameter according to current input state when the 
 * buffer is full. 
 * 
 * We always convert full sequences, so we may signal the end of the output
 * buffer even though we return fewer octets than the maximum given by dl.
 */

{
	uint32_t ch;
	unsigned char *o = (unsigned char *)d;
	unsigned char *e;
	int n;

	for(e = o + dl; UTEST; UADV) {
		ch = UGET;			/* get source value */
		for(n = 0; 			/* search mask table; done if */
		    n < 7 && (ch & ~uchmask[n]);/* no nonmasked bits are set */
		    n++);			/* n=octets to follow 1st one */
		if (e - o <= n) break;		/* no room in dest buffer */
		if (!n) { *o++ = ch; continue; }/* fast case for one octet */
		*o = (unsigned)0xfe << (6 - n);	/* leading pattern in 1st oct.*/
		n *= 6;
		if (n <= 32) *o |= ch >> n;	/* most sig bits in 1st octet */
		o++;
		for(n -= 6; n >= 0; n -= 6) 
			*o++ = 0x80 | ((ch >> n) & 0x3f);
	}
	UDONE;
	return o - (e - dl);
}

